@@ -1,11 +1,12 @@
 __all__ = ['thread',
            'thread_pool',
            'process',
+           'synchronized',
            'ThreadPool',
            'PebbleError',
            'TimeoutError',
            'TaskCancelled']
 
-from .pebble import PebbleError, TimeoutError
+from .pebble import PebbleError, TimeoutError, synchronized
 from .process import process, TaskCancelled
 from .thread import thread, thread_pool, ThreadPool
